/* Copyright (c) 2009, Sean A.O. Harney */

//~--- JDK imports ------------------------------------------------------------
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

public class Jkmt extends JFrame implements ComponentListener {
	private static Jkmt instance = null;
	private static final int WIN_HEIGHT = 700;
	private static final int WIN_WIDTH = 900;

	// singleton pattern
	// fileChooser requires access to the JFrame sothis was needed
	public static Jkmt getInstance() {
		if (instance == null) {
			instance = new Jkmt(); // create the instance of Jiotracer
		}

		return instance;
	}

	public static void main(String[] args) {
		final Jkmt frame = Jkmt.getInstance();


// cant be in constructor since sServerTab wants the singleton of Jkmt
		// already constructed.
	
		frame.consolePanels = new ConsolePanels(new Dimension(WIN_WIDTH - 200, 200));

		JScrollPane ls = new JScrollPane(frame.tabPane, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		frame.splitPane.setLeftComponent(ls);

		JScrollPane rs = new JScrollPane(frame.consolePanels, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		frame.splitPane.setRightComponent(rs);
		frame.add(frame.splitPane);


		frame.setVisible(true);
		frame.splitPane.setDividerLocation(0.5); // must be visible for this to have effect
		frame.startProgressMonitor();
		frame.updateProgress(20); // rendering GUI
		frame.updateProgress(30);

		
		try {
			frame.consolePanels.printlnJkmt("Jkmt v0.1 (Build: " + Revision.svnRevision
					+ " )\n");
		} catch (Exception e) {
		}


		frame.serverTab = new ServerTab();
		frame.tabPane.add("Server", frame.serverTab);
		frame.updateProgress(40);
		frame.optionsTab = new OptionsTab();
		frame.updateProgress(50);
		frame.tabPane.add("Options", frame.optionsTab);
		frame.updateProgress(60);
		frame.dpsTab = new DpsTab();
		frame.tabPane.add(frame.dpsTab.toolName, frame.dpsTab);
		frame.updateProgress(70);
		frame.kmemstatTab = new KmemstatTab();
		frame.tabPane.add(frame.kmemstatTab.toolName, frame.kmemstatTab);
		frame.updateProgress(80);
		frame.ksymstatTab = new KsymstatTab();
		frame.tabPane.add(frame.ksymstatTab.toolName, frame.ksymstatTab);
		frame.updateProgress(90);
		frame.ktraqTab = new KtraqTab();
		frame.tabPane.add(frame.ktraqTab.toolName, frame.ktraqTab);
		frame.updateProgress(95);
		frame.drchkTab = new DrchkTab();
		frame.tabPane.add(frame.drchkTab.toolName, frame.drchkTab);

frame.tabPane.addChangeListener(new ChangeListener() {
            // This method is called whenever the selected tab changes
	             public void stateChanged(ChangeEvent evt) {
	                     JTabbedPane pane = (JTabbedPane)evt.getSource();
	                                       // Get current tab
	                               int sel = pane.getSelectedIndex();
				       if(0 <= sel && sel <= 1) //server,opt
					   frame.consolePanels.tabPane.setSelectedIndex(0); // the All tab
				       if(2 <= sel && sel <= 6) //tooltabs
					   frame.consolePanels.tabPane.setSelectedIndex(sel - 1); // set the console tab to match
				       switch(sel)
				       {
					case 2: //dps
					break;
					case 3: //kmemstat
					break;
					case 4: //ksymstat
					break;
					case 5: //ktraq
					break;
					case 6: //dr_chk
					break;
				       }
	                                                           }
	                                                                });

		frame.updateProgress(100);
	}

	// is set in another file
	AlertsWin alertsWin = null;
	JSplitPane splitPane = null;
	ConsolePanels consolePanels = null;
	DpsTab dpsTab;
	DrchkTab drchkTab;
	KmemstatTab kmemstatTab;
	KsymstatTab ksymstatTab;
	KtraqTab ktraqTab;
	public Net net = null;
	OptionsTab optionsTab;
	int progress;
	ProgressMonitor progressMonitor;

	ServerTab serverTab;

	JTabbedPane tabPane;

	XmlEditorPanel xmlEditorPanel = null;

	// starts progressMonitor at 0 befor Jframe is visible. progress monitor
	// stills shows up
	private Jkmt() {
		super("Innosecc Jkmt");
		setSize(WIN_WIDTH, WIN_HEIGHT);
		addComponentListener(this);
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		progressMonitor = null;
		this.setJMenuBar(new MainMenuBar()); // to this JFrame

		this.tabPane = new JTabbedPane();


		this.splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, false);
		splitPane.setOneTouchExpandable(true);
				try {
			xmlEditorPanel = new XmlEditorPanel();
		} catch (javax.xml.parsers.ParserConfigurationException pe) {
			System.out.println("" + pe.getMessage());
		}
	}

	public void componentHidden(ComponentEvent e) {
	}

	public void componentMoved(ComponentEvent e) {
	}

	/* prevent shrinking the window */
	public void componentResized(ComponentEvent e) {
		if (getWidth() < WIN_WIDTH || getHeight() < WIN_HEIGHT)
			setSize(WIN_WIDTH, WIN_HEIGHT);
	}

	public void componentShown(ComponentEvent e) {
	}

	// shuts down the program
	public void exit() {

		// TODO: do an are you sure dialog
		// todo shutdown iotracers.
		this.setVisible(false);
		System.exit(0);
	}

	// to be called after Jkmt object instantiated that way it is in foreground.
	private void startProgressMonitor() {

		progressMonitor = new ProgressMonitor(this,
				"Initializing Innosecc Jkmt GUI", "", 0, 100);
		progressMonitor.setMillisToPopup(0);
		progressMonitor.setMillisToDecideToPopup(0);

		updateProgress(0);
	}

	private void updateProgress(int percent) {
		this.progress = percent;

		String message = String.format("Completed %d%%.\n", progress);

		progressMonitor.setNote(message);
		progressMonitor.setProgress(progress);

		try {
			Thread.sleep(30);
		} catch (InterruptedException ignore) {
		}
	}


}
