import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import org.w3c.dom.*;

public class KsymstatTab extends ToolTab implements ItemListener,ActionListener,DocumentListener {

	public static String[] lines = {
			"This KMT tool is designed specifically to detect syscall and kernel",
			"function hijacking that rely on what are called trampolines.",
			"",
			"A trampoline is when a hacker replaces the first 6 or 7 bytes of a",
			"kernel function or syscall with code that jumps to an evil replacement",
			"function. Ksymstat will analyze every kernel symbol that is a function",
			"or syscall." };
	JCheckBox mFlagCheckBox = null;

	JCheckBox sysmapCheckBox = null;

	JRadioButton aButton = null;
	JRadioButton sButton = null;
	

	ButtonGroup radioButtons = null;
	JTextField symbolName = null;

	public KsymstatTab() {
		super("ksymstat", lines);


		aButton = new JRadioButton("(note: Linux Only) Scan all symbols (Kernel functions and syscalls)");
		aButton.setActionCommand("abutton");
		aButton.setEnabled(false);
		aButton.addActionListener(this);
		sButton = new JRadioButton("Scan a specific kernel function or syscall (e.g. sys_open)");
		sButton.setActionCommand("sbutton");
		sButton.setEnabled(false);
		sButton.addActionListener(this);
		radioButtons = new ButtonGroup();
		radioButtons.add(aButton);
		radioButtons.add(sButton);

		symbolName = new JTextField(10);
		symbolName.setText("symbol");
		symbolName.setEditable(false);

		this.sysmapCheckBox = new JCheckBox(
				"Use configured system.map file instead of the default /proc/kallsyms");

		sysmapCheckBox.setSelected(false);
		sysmapCheckBox.setEnabled(false);
		sysmapCheckBox.addItemListener(this);

		this.mFlagCheckBox = new JCheckBox(
				"Use /dev/kmt_mem instead of /dev/kmem");
		mFlagCheckBox.setSelected(false);
		mFlagCheckBox.setEnabled(false);
		mFlagCheckBox.addItemListener(this);

		super.ltabCent.add(aButton);

		super.ltabCent.add(sButton);
		super.ltabCent.add(symbolName);

		super.ltabCent.add(sysmapCheckBox);
		super.ltabCent.add(mFlagCheckBox);


		super.finishGui();
	}

	public void buildAndSetArgs() {

		String xpathStr = "/root_element/tool/exec_info[@name=\"ksymstat\"]/@opts";
		Node node = XTree.findNode(xpathStr, jkmt.xmlEditorPanel.xTree.doc);
		if (node == null)
			return;

		String args = " "; // throws out whatever was there and absolutely sets
		// it. for now u can still run kmtd with a modified
		// special args as long as u never mess wit the tab.
		// TODO: perhaps getTextContent() and modify it
		// instead of absolutely setting.

		if (mFlagCheckBox.isSelected()) {
			args += "-M ";
		}

		if (aButton.isSelected()) {
			args += "-a ";
		} else if (sButton.isSelected()) {
		    String sym = symbolName.getText();
		    if ("".equals(sym))
			sym = "emptysymbolname";

			args += "-s " + sym + " ";
		}

		if (sysmapCheckBox.isSelected()) {
			String sysMap = (String) jkmt.optionsTab.systemMapList
					.getSelectedItem(); /*
										 * could alternatively have gotten it
										 * from xml, but this is faster
										 */
			if (sysMap == null)
				return;

			args += " -S " + sysMap;
		}

		XTree.updateNodeContent(node, xpathStr, args); /*
														 * update command line
														 * opts
														 */

		super.refreshCmdArgsLabel();// refresh this tabs jlabel of cmds, TODO:
		// make another method so it doesnt have to
		// get the xml as this one does .

	}

	@Override
	public void disableAll() {
		super.disableAll();
		aButton.setEnabled(false); //cant unselect
		sButton.setEnabled(false); //cant unselect

		sysmapCheckBox.setEnabled(false);
		sysmapCheckBox.setSelected(false);

		mFlagCheckBox.setEnabled(false);
		mFlagCheckBox.setSelected(false);

	}

	// this is optional in tooltab subclasses, if not present supers (an
	// instance of ToolTab) will be called directly.
	@Override
	public void enableAtConnect() {
		super.enableAtConnect(); // overrides super's

		/* populate fields with the values from DOM */
		Node node = XTree.findNode("/root_element/tool/exec_info[@name=\""
				+ toolName + "\"]/@opts", jkmt.xmlEditorPanel.xTree.doc);
		// if(node == null)
		// error
		boolean sF = false, mF = false;
		String curVal = node.getTextContent();
		
		if (curVal.contains("-s "))
		 {
		// parse out next word and setText . The symbol name
		     String[] strs = curVal.split(" ");
		     String s = "unknownsymbol";
		int i;

		     for(i = 0; i < strs.length - 1; i++)
		     {
			if("-s".equals(strs[i]))
			{
			 s = strs[i+1];
			 System.out.println("parsed symbolname: " + s);
			 break;
		     	}
		     }
		sButton.setSelected(true);
		symbolName.setEditable(true);
		symbolName.setText(s);
		 }
		else/* if (curVal.contains("-a")) */
		{
			aButton.setSelected(true);
		symbolName.setEditable(false);
		}


			sF = true;
		if (curVal.contains("-M"))
			mF = true;

		sysmapCheckBox.setSelected(sF);
		mFlagCheckBox.setSelected(mF);

		sysmapCheckBox.setEnabled(true);
		mFlagCheckBox.setEnabled(true);
		aButton.setEnabled(true);
		sButton.setEnabled(true);

		symbolName.getDocument().addDocumentListener(this); // needs to be here, not in constructor
	}

    /** Listens to the radio buttons. */
        public void actionPerformed(ActionEvent e) {
		super.actionPerformed(e);
		if(e.getActionCommand().equals("abutton")) {
		    	symbolName.setEditable(false);
			buildAndSetArgs();
		} else if(e.getActionCommand().equals("sbutton")) {
		    	symbolName.setEditable(true);
			buildAndSetArgs();
		}
	}


	// for the checkbox
	// override's supers
	@Override
	public void itemStateChanged(ItemEvent e) {
		super.itemStateChanged(e); /*
									 * this.itemStateChange was intercepting the
									 * ones for super'sc checkbox. this is the
									 * fix
									 */
		if (e.getSource() != this.sysmapCheckBox
				&& e.getSource() != this.mFlagCheckBox)
			return;

		buildAndSetArgs();
	}

	// for symbolname
	public void insertUpdate(DocumentEvent e) 
	{
		buildAndSetArgs();
	}
	public void removeUpdate(DocumentEvent e)
	{
		buildAndSetArgs();
	}
	public void changedUpdate(DocumentEvent e)
	{
		buildAndSetArgs();
	}
	
}
