/* (C) 2009 Innosecc
 * Author: Sean A.O. Harney
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <signal.h>
#include <errno.h>

#define VAR_RUN_DIR "/var/run"


/* create a pidfile.
 * if this function returns -1 the callee should exit(EXIT_FAILURE).
 * return 0 on success
*/
int pidfile_make(const char *progname)
{
    char fpath[64];
    char buf[20];
    int fd, rr;

    snprintf(fpath, sizeof(fpath), "/%s/%s.pid", VAR_RUN_DIR, progname);

    if ((fd = open(fpath, O_RDWR | O_CREAT | O_EXCL, 0777)) < 0)
      {
	  if (errno != EEXIST)
	    {
		perror("open()");
		return -1;
	  } else
	    {
		/* the file exists already */
		pid_t pid = pidfile_read(fpath);
		if (pid == -1)
		    return -1;
		if (kill(pid, 0) == 0)
		  {
		      fprintf(stderr,
			      "The pid contained in %s is an existing process\n",
			      progname);
		      /*      exit(EXIT_FAILURE); */
		      return -1;
		  }
		/* the pid in the file does not exist, remove file */
		if (unlink(fpath) < 0)
		  {
		      perror("unlink()");
		      return -1;
		  }
		if ((fd = open(fpath, O_RDWR | O_CREAT | O_EXCL, 0777)) < 0)
		  {
		      perror("open()");
		      return -1;
		  }
	    }
      }


      /* the new file has been created at this point.
       * fill it with the pid of current process */
      snprintf(buf, sizeof(buf), "%d\n", getpid());

      if((rr = write(fd, buf, strlen(buf))) < 0)
      {
	  perror("write()");
	  close(fd);
	  return -1;
      }

	close(fd);
	return 0;
}

pid_t pidfile_read(const char *fpath)
{
    int fd, rr;
    char buf[20];
    pid_t ret;

    if ((fd = open(fpath, O_RDONLY)) < 0)
      {
	  perror("open()");
	  return -1;
      }

    if ((rr = read(fd, buf, sizeof(buf) - 1)) < 0)
      {
	  perror("read()");
	  close(fd);
	  return -1;
      }
    buf[rr] = '\0';

    if ((ret = strtoul(buf, NULL, 10)) <= 0)	/* base 10 */
      {
	  fprintf(stderr, "Unable to parse pidfile value: %s\n", buf);
	  close(fd);
	  return -1;
      }

    close(fd);
    return ret;
}
