#include "kmond.h"

#define DAEMON 1								/* 0 or #undef to make kmond start without becoming daemon. Compile-time option */

void start_kmtd ();

#define KMTD "/opt/kmt/sbin/kmtd"
#define KMTD_DEFAULT_ARGS "-c /opt/kmt/kmt_config/kmt.conf"
#define XDELIM ": "

char cmdargs[512];
char **xargs;
pid_t global_pid = -1;
pid_t global_kmtd_temp_pid = -1;
int kmtd_need_restart = 0;			/* TODO: make bool */

void (*KMTLog_msg) (char *, ...);
void KMTLog_err (char *, ...);
void sigchld_handler (int signum);

int first_time = 1;

int
main (int argc, char **argv)
{
	int i = 0, n;
	int tokens, ret, status;
	pid_t pid;
	struct sigaction sa;

	/* KMTLog_msg is the same as KMTLog_err so for simplicity */
	/* We just use a function ptr */
	KMTLog_msg = KMTLog_err;

	sa.sa_handler = sigchld_handler;
	sa.sa_flags = 0;
	sigemptyset (&sa.sa_mask);
	sigaction (SIGCHLD, &sa, NULL);


	n = snprintf (cmdargs, sizeof (cmdargs), "%s", KMTD);
	/* n does not include the trailing '\0' == strlen(cmdargs) */

	if (argc == 1)
	{
		cmdargs[n++] = ' ';
		strncat (cmdargs, KMTD_DEFAULT_ARGS, sizeof (cmdargs) - 1 - n);
	}

	argv += 1;
	argc -= 1;
	while (argc--)
		snprintf (&cmdargs[strlen (cmdargs)],
							sizeof (cmdargs) - strlen (cmdargs), ":%s", argv[i++]);
	tokens = exargs (&xargs, XDELIM, cmdargs);
	if (tokens < 0)
	{
		KMTLog_err ("kmond failed to start %s", KMTD);
		exit (-1);
	}

	printf ("KmonD v1.0 - (C) Innosecc.com 2009\n");

	for (n = 0; n < tokens; n++)	/* changed from tokens+1 to tokens because it was priting (null), this will segfault on many unix */
		printf ("%s ", xargs[n]);
	printf ("\n");
	/* NOTE: might as well have printed out cmdline, however it is
	 * already coded and is a good test of the exargs() */

#if DAEMON
	/* changed 2nd arg to 1 so that it does not redirect stdout,stderr to /dev/null */
	/* it was causing problems with seeing the pidfile errors which need to be shown */
	if (daemon (1, 1) == -1)
	{
		KMTLog_err ("Fatal error in daemonizing kmond, daemon() %s",
								strerror (errno));
		exit (-1);
	}
#endif /* DAEMON */

	/* now that went daemon and have new pid, create the pid file - sean */

	if (pidfile_make ("kmond") == -1)
	{
		KMTLog_err ("Could not make pidfile, exiting");
		exit (-1);
	}


	/* Execute kmtd for the first time */
	start_kmtd ();
	first_time = 0;

#if DAEMON
	/* now complete the daemonizing procedure by closing stdout,stderr,stdin */
	/* Moved as per Seans suggestion, down below fork and exec (start_kmtd()) */
	fclose (stdin);
	fclose (stdout);
	fclose (stderr);
#endif /* DAEMON */


	for (;;)
	{
		usleep (1000000);



		/* Does process still exist? */
		if (kill (global_pid, 0) == 0)
		{
			/* the pid is an existing process */
			continue;
		}
		else
		{
			if (errno != ESRCH)
				continue;
			/* else it doesn't exist, or is a zombie */
		}

		KMTLog_err ("kmond msg: kmtd appears to have stopped running.");
		start_kmtd ();
	}

}




/* fork and exec kmtd */
void
start_kmtd ()
{
	int ret, status;
	pid_t pid;


	pid = fork ();

	if (pid < 0)
	{
		KMTLog_err ("kmond failed to start %s -> fork() error: %s", KMTD,
								strerror (errno));
		exit (-1);
	}

	global_kmtd_temp_pid = pid;


	if (!pid)											/* child */
	{
		ret = execv (KMTD, xargs);
		if (ret < 0)
		{
			KMTLog_err ("kmond failed to start %s (execv failure: %s)",
									KMTD, strerror (errno));
			exit (-1);								/* TODO: this error is not fatal to the parent, only to the child here */
		}
		/* else execv never returns */
	}
	/* else parent */
	sleep (2);										

/* sleep 2 seconds to give kmtd time to make it's var run file. If this race condition fails, the worse that will happen is kmond will think kmtd has died and restart it, it is very unlikely it would fail */
	global_pid = pidfile_read ("/var/run/kmtd.pid");
	if (global_pid < 0)
	{
		KMTLog_err
			("kmond: Failure in retrieving PID of kmtd, internal error. Restart kmond manually\n");

		exit (-1);
	}

	if (first_time == 0)
		KMTLog_msg ("kmond msg: kmtd restarting.");


}

void
sigchld_handler (int signum)
{
	pid_t cpid;
	int status;


	/* Reap All child process. If none then don't block */
	while ((cpid = waitpid (-1, &status, WNOHANG)) > 0)
	{
		if (cpid == global_kmtd_temp_pid && global_kmtd_temp_pid != -1)
		{
			// if it does not finish with exit(0) something went wrong 
			if (WIFEXITED (status) && (WEXITSTATUS (status) == EXIT_SUCCESS))
			{
				if (first_time == 0)
					KMTLog_msg ("kmond msg: kmtd daemon executed cleanly.");
			}
			else
			{
				KMTLog_err
					("kmond: Could not execute the kmtd daemon cleanly, internal error. Restart kmond manually.");
				exit (-1);
			}

		}
		else
			KMTLog_msg ("kmond msg: Reaped unknown process: %ld", cpid);	/* not sure if it will ever get here. But want to see if it logs it happening */

	}
	//THIS CODE IS USELESS RIGHT NOW SINCE kmtd is not a child, it is a grandchild, so we rnt getting sigchlds for it when it gets kill -9'ed etc!

	/* else  if (cpid == global_pid && global_pid != -1)
	   {
	   // only want to print out the ones for the main kmtd pid (the one in the var run file 
	   if (WIFEXITED(status))
	   KMTLog_msg
	   ("kmond msg: kmtd (%d) terminated with exit status %d",cpid,
	   WEXITSTATUS(status));
	   else if (WIFSIGNALED(status))
	   {
	   int sign = WTERMSIG(status);
	   KMTLog_msg("kmond msg: kmtd (%d) terminated by signal %d: %s",cpid
	   ,sign, strsignal(sign));
	   if (WCOREDUMP(status)) // only if signaled should this be used! 
	   KMTLog_msg
	   ("kmond msg: kmtd (%d) crashed and dumped core file", cpid);
	   }

	   else
	   {
	   // we don't know the reason
	   KMTLog_msg
	   ("kmond msg: kmtd (%d) exited by unknown means", cpid);
	   }
	 */
	/* end while */

}
