/*
 * kmt_dr (C) innosecc.com 2009
 * insmod kmt_dr.ko
 */

#include <linux/init.h>
#include <linux/module.h>
#include <linux/kernel.h>				/* printk() */
#include <linux/slab.h>					/* kmalloc() */
#include <linux/fs.h>						/* everything... */
#include <linux/errno.h>				/* error codes */
#include <linux/types.h>				/* size_t */
#include <linux/proc_fs.h>
#include <linux/fcntl.h>				/* O_ACCMODE */

#include <linux/mm.h>						/* for VM_EXECUTABLE */
#include <linux/mount.h>				/* for mnt_get */
#include <linux/moduleparam.h>
#include <linux/sched.h>

#include <asm/system.h>					/* cli(), *_flags */
#include <asm/uaccess.h>				/* copy_from/to_user */


#define BD_FLAG 0x2000 /* For dr6 */
#define GD_FLAG 0x2000 /* For dr7 */

/* GET_DR0(dr) volatile just incase */
#define GET_DR0(x) __asm__ __volatile__("movl %%dr0, %0" : "=r" (x));
#define GET_DR1(x) __asm__ __volatile__("movl %%dr1, %0" : "=r" (x));
#define GET_DR2(x) __asm__ __volatile__("movl %%dr2, %0" : "=r" (x));
#define GET_DR3(x) __asm__ __volatile__("movl %%dr3, %0" : "=r" (x));
#define GET_DR6(x) __asm__ __volatile__("movl %%dr6, %0" : "=r" (x));

int kmt_dr_open (struct inode *inode, struct file *filp);
ssize_t kmt_dr_read (struct file *filp, char *buf, size_t count, loff_t * f_pos);

/* Global variables of the driver */
/* Major number */
int kmt_dr_major = 251;
unsigned long kmtd_inode = 0;

int opened = 0;

module_param (kmt_dr_major, int, 0);
module_param (kmtd_inode, long, 0);

struct file_operations kmt_dr_fops = {
read:kmt_dr_read,
open:kmt_dr_open,
		/* Make sure to NULL initialize unused ops */
		/*
		   readdir: NULL,
		   ioctl: NULL,
		   mmap: NULL,
		   fsync: NULL,
		   lock: NULL,
		   fasync: NULL,
		   flush: NULL,
		   release: NULL
		 */
};


/* doesn't care what count() written, TODO: allow it to read byte by byye upto count */
ssize_t
kmt_dr_read (struct file * filp, char *buf, size_t count, loff_t * f_pos)
{
	int i;
	unsigned long dr[8];
	long regstate;

	if(opened == 0)
		return -EAGAIN;
	opened = 0;

	GET_DR0(dr[0]);
	GET_DR1(dr[1]);
	GET_DR2(dr[2]);
	GET_DR3(dr[3]);
	GET_DR6(dr[6]);
	
	/* If someone is accessing the debug registers */
	/* Lets set them straight! */
	if (dr[6] & BD_FLAG)
	{
		regstate = dr[6];
		/* Modify register state so BD is not set */
		regstate &= ~BD_FLAG;
		/* Reset register state of db6 */
		 __asm__ __volatile__ ("movl %0, %%db6" : : "r" (regstate));
			
		
	}

	for(i = 0; i < 4; i++)
		printk("kmt_dr: dr%d = %ld\n", i, dr[i]);


	/* Copy text data to user space */
	if (copy_to_user (buf, &dr, sizeof(dr)))
		return -EFAULT;


	/* Increase offset */
	*f_pos += sizeof(dr);

	return sizeof(dr);

}

int
kmt_dr_open (struct inode *_inode, struct file *filp)
{
	// struct task_struct *task = NULL;
	pid_t pid = current->pid;
	pid_t pgid = current->group_leader->pid;
	char *comm = current->group_leader->comm;

	struct dentry *dentry = NULL;
	struct vfsmount *mnt = NULL;
	struct vm_area_struct *vma = NULL;
	unsigned long ino = 0;

	if (kmtd_inode == 0)
	{
		opened = 1;
		return 0;										//success, not using optional security
	}

	down_read (&current->group_leader->mm->mmap_sem);


	vma = current->group_leader->mm->mmap;
	while (vma)
	{
		if ((vma->vm_flags & VM_EXECUTABLE) && vma->vm_file)
			break;
		vma = vma->vm_next;
	}

	if (vma)
	{
		mnt = mntget (vma->vm_file->f_path.mnt);
		dentry = dget (vma->vm_file->f_path.dentry);
	}

	up_read (&current->group_leader->mm->mmap_sem);

	if (dentry)
	{
		ino = dentry->d_inode->i_ino;
		/* inode is dentry->d_inode */
	}

	//generally comm same as argv[0], TODO: tes tif modifying argv[0] effects.

	printk ("kmt_dr: current->pid = %d, current->group_leader->pid = %d\n",
					pid, pgid);
	printk ("kmt_dr: group leader comm = %s, inode number = %ld\n", comm, ino);


	if (0 != kmtd_inode && ino != kmtd_inode)
	{
		/* option kmtd_inode= was given and it doesnt match inode of group leader. TODO; make it look for inode of group leader */
		return -EACCES;
	}

	/* Success */
		opened = 1;
	return 0;
}


int init_kmt_dr (void)
{	int result;

	/* Registering device */
	if ((result = register_chrdev (kmt_dr_major, "kmt_mem", &kmt_dr_fops)) < 0)
	{
		printk ("kmt_mem: Unable to get major number %d\n", kmt_dr_major);
		return result;
	}

	printk ("kmt_dr: Registered device kmt_mem major: %d\n", kmt_dr_major);
	
	printk ("Setting GD flag in dr7\n");

	/* We need GD_FLAG set so that we can detect any future mov's (read/writes) to */
	/* The debug registers */
	 __asm__ __volatile__ ("movl %0, %%db7" : : "r" (GD_FLAG));

	return 0;
}

void close_kmt_dr (void)
{
	printk ("kmt_dr: Removing kmt_dr\n");
	unregister_chrdev (kmt_dr_major, "kmt_dr");
}

module_init (init_kmt_dr);
module_exit (close_kmt_dr);
