#include "kmtd.h"

extern char global_alert_buffer[8192 * 8];

void KMTLog_alert(char *fmt, ...)
{
        FILE *fd;
        char time_buf[255];
        time_t clock;
        struct tm *tm;
        char errmsg[4096];
        va_list va;

        if ((fd = fopen("/var/log/kmt/kmtd.alerts", "a+")) == NULL)
        {
                NotifyUsers("ALERT! Unable to log to /var/log/kmt/kmtd.alerts: %s\n", strerror(errno));
                return;
        }

        va_start (va, fmt);
        vsnprintf (errmsg, sizeof(errmsg), fmt, va);
        va_end (va);

        time (&clock);
        tm = gmtime (&clock);
        setlocale (LC_TIME, "C");
        strftime (time_buf, sizeof (time_buf), "%A, %B %e, %Y, %H:%M:%S", tm);

        fprintf(fd, "%s %s\n", time_buf, errmsg);
        fflush(fd);

        fclose(fd);
}

/* This logs KMT failures to kmtd.log (or char *logfile) */
/* I.E if an Alert fails, or if a file can't be opened */
void KMTLog_err(char *fmt, ...)
{
        FILE *fd;
        char time_buf[255];
        time_t clock;
        struct tm *tm;
	char errmsg[4096];
	va_list va;

        if ((fd = fopen("/var/log/kmt/kmtd.log", "a+")) == NULL)
        {
                NotifyUsers("ALERT! Unable to log to /var/log/kmt/kmtd.log: %s\n", strerror(errno));
	    	return;
        }

	va_start (va, fmt);
        vsnprintf (errmsg, sizeof(errmsg), fmt, va);
        va_end (va);
        
	time (&clock);
        tm = gmtime (&clock);
        setlocale (LC_TIME, "C");
        strftime (time_buf, sizeof (time_buf), "%A, %B %e, %Y, %H:%M:%S", tm);

        fprintf(fd, "%s %s\n", time_buf, errmsg);
        fflush(fd);

        fclose(fd);
}

/* This logs to kmtd.log, noting that a specific tool found an anomaly */
void KMTLog(char *tool, char *logfile)
{
	FILE *fd;
        char time_buf[255];
        time_t clock;
        struct tm *tm;

        if ((fd = fopen(logfile, "a+")) == NULL)
        {
                NotifyUsers(strerror(errno));
	    	return;
        }

        time (&clock);
        tm = gmtime (&clock);
        setlocale (LC_TIME, "C");
        strftime (time_buf, sizeof (time_buf), "%A, %B %e, %Y, %H:%M:%S", tm);
	
	fprintf(fd, "%s [%s] -> Succeeded in detecting anomaly\n", time_buf, tool);
	fflush(fd);

	fclose(fd);
}
	
/* This logs whatever is in the global_alerts_buffer */
/* To the specified tool */
void Write2Log(char *logfile)
{
	FILE *fd;
	char time_buf[255];
	time_t clock;
  	struct tm *tm;

	if ((fd = fopen(logfile, "a+")) == NULL)
	{
		NotifyUsers(strerror(errno));
	    	return;
	}
	
	time (&clock);
 	tm = gmtime (&clock);
  	setlocale (LC_TIME, "C");
  	strftime (time_buf, sizeof (time_buf), "%A, %B %e, %Y, %H:%M:%S", tm);

	fprintf(fd, "\n-----[%s]-----\n", time_buf);
	fprintf(fd, "%s\n", global_alert_buffer);
	fflush(fd);

	fclose(fd);
}


