#include "kmtd.h"

#define EMAIL_STR "/usr/lib/sendmail"
#define ALERT_TMP "/tmp/.kmtd_alert"

extern char global_alert_buffer[8192 * 8];
extern char *email[255];
extern int email_count;
extern char host_OS[255];
extern uint32_t host_ip;
extern char hostname[255];
extern struct opts opts;

void ReportIntrusion(int tool)
{
	uint8_t *mem;
	int fd;
	struct stat st;
	FILE *fp;
	extern char global_zip_passwd[32];

	/* 
	TODO: reply-to: 
	 we can let the mail program add the To: as its doing per recipeint
	*/

	char boundary[33];//32char and a \0
	int i, r;

	if ((fp = fopen(ALERT_TMP, "w+")) == NULL)
	{
		KMTLog_err("fopen() failed upon opening message to Notify Users\n");
		return;
	}	

	srand(time(NULL));
	for(i = 0; i < 32; i++)
	{
    		do {
			r = (rand() % 126) + '0';
    		   }    while(!isalnum(r)); //keep trying till we get an alnum
    		
		boundary[i] = r; 
	}
	boundary[32] = '\0';

	fprintf(fp, 
	"Subject: %s\n"
	"MIME-Version: 1.0\n"
	"Content-Type: multipart/mixed; boundary=\"%s\"\n\n" // \\n\n seperates heder from email body
	,"KMT ALERT", boundary);

	switch (tool)
	{
	    	case TOOL_T_FLAG:
			fprintf(fp,
			"This is a test alert generated by kmtd -t\n");
		break;
		case TOOL_KMEMSTAT:
			fprintf(fp,
			"ALERT! This is an important message from KMTD (Your KMT Daemon).\n"
			"A modification to kernel memory has been found by the tool 'KmemStat'\n"
			"This tool analyzes kernel memory, and compares it to the actual kernel\n"
			"image that is loaded at runtime. KmemStat is aware that Linux systems\n"
			"perform patching at runtime which can create discrepencies, and because\n"
			"of this, KmemStat carefuly dissects special sections the in kernel image\n"
			"to tell which parts of kernel memory these patches have been applied.\n"
			"In other words... if you are getting this Alert, then KmemStat has determined\n"
			"that illegal modifications to kernel memory have been applied. This is\n"
			"classified as kernel malware. KmemStat checks for ANY illegal modifications to\n"
			"kernel memory. For more details audit /var/log/kmt/kmemstat.log (And all others)\n\n"
			"Our recommendation is to begin performing forensics immediately... the hacker\n"
			"is still present, and is not aware that you know he is present. Move production\n"
			"to another server, and remove this one from the network ASAP. Here is information\n"
			"about the affected machine...\n\n\n"
			"IP Address:       %s\n"
			"Hostname:         %s\n"
			"Operating System: %s\n"
			"Type of kernel rootkit: Unknown\n"
			"Tool that discovered malware: KmemStat\n"
			"KmemStat Summary:\n\n"
			"%s\n\n", inet_ntoa(host_ip), hostname, host_OS, global_alert_buffer);
			break;
		case TOOL_KSYMSTAT:

			fprintf(fp,
			"ALERT! This is an important message from KMTD (Your KMT Daemon).\n"
			"Using the tool KsymStat, I have discovered that one or more kernel\n"
			"functions or syscalls have been hijacked. This indicates that a hacker\n"
			"is present, and has weakened your kernel, fooling it to execute code\n"
			"that is not native to the System. This code works in the hackers best\n"
			"interest (by making him invisible) but immediately destabilizes the entire\n"
			"System. For more details audit /var/log/kmt/ksymstat.log (And all others)\n\n"
			"Our recommendation is to begin performing forensics immediately... the hacker\n"
                        "is still present, and is not aware that you know he is present. Move production\n"
                        "to another server, and remove this one from the network ASAP. Here is information\n"
                        "about the affected machine...\n\n\n"
                        "IP Address:       %s\n"
                        "Hostname:         %s\n"
                        "Operating System: %s\n"
                        "Type of kernel rootkit: Unknown\n"
                        "Tool that discovered malware: KmemStat\n"
                        "KsymStat Summary:\n\n"
                        "%s\n\n", (char *)inet_ntoa(host_ip), hostname, host_OS, global_alert_buffer);
                        break;
		case TOOL_DPS:
			fprintf(fp,
			"ALERT! This is an important message from KMTD (Your KMT Daemon).\n"
			"Using the tool DPS, I have discovered that the system_call() handler\n"
			"has been illegally modified in a way that indicates a kernel rootkit\n"
			"is present (Along with a hacker). The system_call() handler has suddenly\n"
			"been patched to use a sys_call_table[] that does not match the one as\n"
			"described in your System.map file. (I check to make sure the contents\n"
			"of this file have not been modified everytime I run tools that need it\n"
			"and will notify you in a seperate email if I see that it has changed.\n"
			"If you are seeing this email, and do not see an email notifying you that\n"
			"the System.map file has changed, then you can be certain that this\n"
			"machine has been hacked. For more details audit /var/log/kmt/dps.log (And all others)\n"
			"Our recommendation is to begin performing forensics immediately... the hacker\n"
                        "is still present, and is not aware that you know he is present. Move production\n"
                        "to another server, and remove this one from the network ASAP. Here is information\n"
                        "about the affected machine...\n\n\n"
                        "IP Address:       %s\n"
                        "Hostname:         %s\n"
                        "Operating System: %s\n"
                        "Type of kernel rootkit: Unknown\n"
                        "Tool that discovered malware: KmemStat\n"
                        "DPS Summary:\n\n"
                        "%s\n\n", inet_ntoa(host_ip), hostname, host_OS, global_alert_buffer);
                        break;	
		case TOOL_KTRAQ:
			fprintf(fp,
			 "ALERT! This is an important message from KMTD (Your KMT Daemon).\n"
			 "Using the tool Ktraq, I have discovered that one or more syscall\n"
			 "has been hijacked. The sys_call_table is periodically compared to\n"
			 "their corresponding symbols in your System.map file. This file should\n"
			 "never change. If this file DOES change, then I will notify you promptly.\n"
			 "Hijacked syscalls are the most common method for rootkits to compromise\n"
			 "the kernels integrity. For more details audit /var/log/kmt/ktraq.log (And all others)\n"
			 "Our recommendation is to begin performing forensics immediately... the hacker\n"
                         "is still present, and is not aware that you know he is present. Move production\n"
                         "to another server, and remove this one from the network ASAP. Here is information\n"
                         "about the affected machine...\n\n\n"
                         "IP Address:       %s\n"
                         "Hostname:         %s\n"
                         "Operating System: %s\n"
                         "Type of kernel rootkit: Unknown\n"
                         "Tool that discovered malware: KmemStat\n"
                         "Ktraq Summary:\n\n"
                         "%s\n\n", inet_ntoa(host_ip), hostname, host_OS, global_alert_buffer);
                        break;	
		case TOOL_DR_CHK:
			fprintf(fp,
			"ALERT! This is an important message from KMTD (Your KMT Daemon).\n"
			"Using the tool dr_chk, I have discovered that your INT1 handler\n"
			"has been patched with a malicious do_debug call. This means that\n"
			"an infamous ia32 debug register rootkit is present on your system.\n"
			"Our recommendation is to begin performing forensics immediately...\n"
			"the hacker is still present, and is not aware that you know this.\n"
			"Move production to another server, and remove this one from the network\n"
			"ASAP. Here is information about the affected machine...\n\n\n"
			"IP Address:       %s\n"
                        "Hostname:         %s\n"
                        "Operating System: %s\n"
                        "Type of kernel rootkit: Unknown\n"
                        "Tool that discovered malware: KmemStat\n"
                        "Ktraq Summary:\n\n"
                        "%s\n\n", inet_ntoa(host_ip), hostname, host_OS, global_alert_buffer);
			break;

		default:
			return;		
			
	}
	
	char zip_file[255];

	if (compress(global_zip_passwd, zip_file) < 0)
		goto skip_attachment;
	
	if ((fd = open(zip_file, O_RDONLY)) < 0)
	{
		KMTLog_err("Unable to open zip file %s for alert attachment: %s\n", zip_file, strerror(errno));
		goto skip_attachment;
	}

	if (fstat(fd, &st) < 0)
	{
		KMTLog_err("Unable to stat zip file %s for alert attachment: %s\n", zip_file, strerror(errno));
		goto skip_attachment;
	}

	mem = mmap(NULL, st.st_size, PROT_READ, MAP_PRIVATE, fd, 0);
	if (mem == MAP_FAILED)
	{
		KMTLog_err("Unable to mmap zip file %s for alert attachment: %s\n", zip_file, strerror(errno));
		goto skip_attachment;
	}



	int outz = st.st_size * 2.5 + 10; // a little bigger than the biggest possible b64 encoding.
	char *out = malloc(outz);
	
	if(out == NULL)
	{
		perror("malloc()");	
		KMTLog_err("malloc() failed: %s\n", strerror(errno));
		goto skip_attachment;
	}

	if(base64_encode(mem, st.st_size, out, outz) == 0)//failed
	{
		free(out);
		KMTLog_err("base64_encode() failed\n");
		goto skip_attachment;
	}

	char *fname = strrchr(zip_file, '/');
	if(fname == NULL) // was not an absolute path
    		fname = zip_file;
    	else 
		fname++; //fname now points to the file name


	fprintf(fp,
		"--%s\n"
		"Content-Type: application/zip\n"
		"Content-Disposition: attachment; filename=\"%s\"\n"
		"Content-Transfer-Encoding: base64\n\n"
		, boundary, fname);


	
// must be under 998 octets
#define MIME_LINELEN 60
		//TODO: make this write in chunks for efficiency
		for(i = 0; out[i] != '\0'; i++)
		{
		    fwrite(&out[i],1,1,fp);
			if(i % MIME_LINELEN == MIME_LINELEN - 1) // smtp requires a enwline after at most 998 octets
			    fprintf(fp, "\n");
		}
		
	fprintf(fp, "\n--%s--\n", boundary);
	fflush(fp);
	fclose(fp);
	free(out);
	NotifyUsers(NULL);
	return;

	skip_attachment:

	fflush(fp);
	fclose(fp);
	if(out != NULL)
	free(out);
	NotifyUsers(NULL);
	munmap(mem, st.st_size);

}

//sends the already build temp file
void NotifyUsers(char *fmt, ...)
{

	extern char mailutil[255];
	int len = 255 + strlen(mailutil) + strlen(ALERT_TMP) + 2;
	int i;
	char *p, *tmp, buf[512], msg[8192 * 4], mbuf[8192 * 4];
	FILE *pd, *fd;
	extern int SendAlerts;
	struct stat st;
	time_t clock;
        struct tm *tm;
        va_list va;
	
	if (!fmt)
		goto send_alerts;

 	va_start (va, fmt);
        vsnprintf (msg, sizeof(msg), fmt, va);
        va_end (va);

	/* NEW CODE THAT STORES ALERTS IN kmtd.alert */ 
	/* If there is a message passed in char *msg */
	if (fmt)
		KMTLog_alert(msg);
	else
	{
		/* Otherwise let's get what's in the /tmp/alert file */
		if ((fd = fopen(ALERT_TMP, "r")))
			if (stat(ALERT_TMP, &st) == 0)
			{
				fread((char *)mbuf, 1, st.st_size, fd);
				KMTLog_alert(mbuf);
				fclose(fd);
			}
	}		
	/* ----------------------------------------- */					
	
	send_alerts:	
			
	if (SendAlerts == 0)
		return;

	tmp = alloca(len);
	memset(tmp, 0, sizeof(tmp));

	/* Run mail --user=KMD admin@innosecc.com < /tmp/kmtd_alert */
	/* against all email addresses that are in char *email[] */
	for (i = 0; i < email_count; i++)
	{
		if (opts.debug)
			printf("[+] Email address: %s\n", email[i]);
		snprintf(tmp, len, "%s %s < %s", mailutil, email[i], ALERT_TMP);

		if (opts.debug)
			printf("[+] Notifying users, executing string: %s\n", tmp);
	
		if ((pd = popen(tmp, "r")) == NULL)
		{
			perror("popen()");
			continue;
		}
		else
		{
			fgets(buf, sizeof(buf), pd);
			if (strstr(buf, "error"))
				KMTLog_err("Alert failure with mail client: %s", buf);
					
				if (opts.debug)
					printf("[+] Mail: %s\n", buf);
			pclose(pd);
		}
		memset(tmp, 0, sizeof(tmp));
	}

}

