#include "kmtd.h"

#define FIND_KIMAGE_LINUX "find /usr/src -maxdepth 2 -type f -name vmlinux"
#define FIND_KIMAGE_FREEBSD "find /boot -type f -name kernel"

extern struct opts opts;
extern struct config_info config;

/* Function to locate uncompressed kernel image/s */
char ** get_kimage_paths(int *count, int *len)
{
	FILE *pd;
	char cmd[255], buf[255]; 
	char **kimage_paths, *p;
	int c = 0;

	if ((kimage_paths = (char **)calloc(32, sizeof(char *))) == NULL)
		return NULL;
		
#ifdef __linux__
	strcpy(cmd, FIND_KIMAGE_LINUX);
#else
	strcpy(cmd, FIND_KIMAGE_FREEBSD);
#endif
	*len = 0;
	*count = 0;

	if ((pd = popen(cmd, "r")) == NULL)
		return NULL;
	
	if (config.kernel_image)
	{ 
		if ((kimage_paths[c++] = strdup(config.kernel_image)) == NULL)
			return NULL;
		*(len) += strlen(config.kernel_image);
		*count = 1;
	}
	/* Here we get a listing of kernel images */
	while (fgets(buf, sizeof(buf), pd))
	{
		buf[strlen(buf) - 1] = '\0';
		
		/* Confirm its an ELF EXEC file */
		if (IsElf(buf))
		{
			if (opts.debug)
				printf("[+] Found image: %s\n", buf);
				
			if ((p = strchr(buf, '\n')))	
				*p = '\0';
			if (!strcmp(buf, config.kernel_image))
                        	continue;
			*(len) += strlen(buf);
			if ((kimage_paths[c++] = strdup(buf)) == NULL)
				return NULL;
			*count = c;
		}
	
	}
	pclose(pd);
	if (*count == 1 && opts.debug)
		printf("kimage_paths[0]: %s\n", kimage_paths[0]);
	if (*count)
		return kimage_paths;

	/* Shouldn't get here */ 
	return NULL;

}
	
char ** get_sysmap_paths(int *count, int *len)
{
	
	struct utsname uts;
	char *sysmap_release;
	char *sysmap_modules;
	char *sysmap_bootdir;
	char *sysmap_srcdir;
	char *sysmap_inroot;
	int ret;
	FILE *fd, *pd;
	
	int j;
	char **system_map_paths;
	char buf[255], line[512], *p;
	char *current_path;
	
	if (uname(&uts) < 0)
		return NULL;
		
	current_path = config.system_map;

	if (opts.debug)
		printf("getmaps() currently set path: %s\n", current_path);
	
	if ((sysmap_modules = (char *)malloc(255)) == NULL)
                return NULL;
	
	if ((system_map_paths = (char **)calloc(32, sizeof(char *))) == NULL)
		return NULL;
	
	snprintf(sysmap_modules, sizeof(sysmap_modules), "/lib/modules/%s/System.map", uts.release);
	
	/* init len */
	*len = 0;
	*count = 0;
	
	if (current_path[0] != '\0')
		system_map_paths[(*count)++] = strdup(current_path);
		*len += strlen(current_path);
		/*NOTE: current is added even if file doesn't exist */
	
	if (strcmp(current_path, sysmap_modules) != 0)
		if ((fd = fopen(sysmap_modules, "r")) != NULL)
        	{
			fclose(fd);
			*len += strlen(sysmap_modules);
			system_map_paths[(*count)++] = strdup(sysmap_modules);
			free(sysmap_modules);
        	}
	if (strcmp(current_path, "/usr/src/linux/System.map") != 0)
		if ((fd = fopen("/usr/src/linux/System.map", "r")) != NULL)
		{
			fclose(fd);
			*len += strlen("/usr/src/linux/System.map");
			system_map_paths[(*count)++] = strdup("/usr/src/linux/System.map");
		}	
	if (strcmp(current_path, "/System.map") != 0)
		if ((fd = fopen("/System.map", "r")) != NULL)
		{
                	fclose(fd);
			*len += strlen("/System.map");
                	system_map_paths[(*count)++] = strdup("/System.map");
        	}
	
	if ((pd = popen("/bin/ls /boot/*.map*", "r")) != NULL)
	{
		while (fgets(buf, sizeof(buf), pd))
		{
			/* We don't want to add the same path twice (buf will have newline)*/
			if (!strncmp(buf, current_path, strlen(buf)-1))
				continue;

			*(char *)strchr(buf, '\n') = '\0';

			/* These are any System.maps in /boot that we missed */
			system_map_paths[(*count)++] = strdup(buf);
			*len += strlen(buf);
		}
		pclose(pd);
	}

	return system_map_paths;
	
}


