/*
 *  KMTD (C) Innosecc.com 2009.
 *  The Kernel malware tracker Daemon. This software abides by the KMTD protocol
 *  as specified in KMTD-PROTOCOL file.
 *  Author of Daemon: Ryan@Innosecc.com
 *  Author of Client: Sean@Innosecc.com
 *  
 */

#include "kmtd.h"

int
ip_connect (unsigned long ip, int port)
{
	int sock, n;
	struct sockaddr_in saddr;
  	unsigned long sflag = 0;

  	if ((sock = socket (AF_INET, SOCK_STREAM, 0)) == -1)
    		return -1;

  	memset (&saddr, 0, sizeof (struct sockaddr_in));

  	saddr.sin_family = AF_INET;
  	saddr.sin_port = htons (port);
  	saddr.sin_addr.s_addr = ip;

  	sflag = fcntl (sock, F_GETFL, 0);
  	fcntl (sock, F_SETFL, sflag | O_NONBLOCK);

  	if ((n = connect (sock, (struct sockaddr *) &saddr, sizeof (struct sockaddr_in)) < 0))
    		if (errno != EINPROGRESS)
      			return -1;

  	return sock;
}

/* See if sock is ready for reading */
int
socket_status(int sfd, int secs)
{
	int status, n;
  	u_long len = 4;
  	fd_set rset;
  	struct timeval tval;

	FD_ZERO (&rset);
  	FD_SET (sfd, &rset);

  	tval.tv_sec = secs;
  	tval.tv_usec = 0;

  	select (sfd + 1, &rset, NULL, NULL, secs ? &tval : NULL);

        if (FD_ISSET (sfd, &rset))
               return 1;
	
	return 0; 
}

/* Returns socket */
int bind_connection(void)
{
	
	int sockfd, ss, len, bytes;
	int newfd, opt;
	socklen_t sin_size;
	unsigned long sflag;
	extern int interactive;
	extern int port;

  	struct sockaddr_in my_addr;
  	struct sockaddr_in their_addr;
	
	fd_set wset, rset;

	memset(&my_addr, 0, sizeof(struct sockaddr_in));

  	if ((sockfd = socket (AF_INET, SOCK_STREAM, 0)) == -1)
    	{
      		perror ("socket");
      		exit (-1);
    	}

	/* We need this for accept() */
	sflag = fcntl (sockfd, F_GETFL, 0);
  	fcntl (sockfd, F_SETFL, sflag | O_NONBLOCK);
	
	opt = 1;
	if (setsockopt (sockfd, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof (opt)) < 0)
	{
		perror("setsockopt");
		exit(-1);
	}

	my_addr.sin_family = AF_INET;
  	my_addr.sin_port = htons (port);
  	my_addr.sin_addr.s_addr = INADDR_ANY;
  	bzero (&(my_addr.sin_zero), 8);

	if (interactive)
		printf("[!] Bind on port: %d\n", port);

  	if (bind (sockfd, (struct sockaddr *) &my_addr, sizeof (struct sockaddr)) == -1)
    	{
      		perror ("bind");
      		exit (-1);
   	}
 
	// fcntl(sockfd, F_SETFL, O_NONBLOCK);

	return sockfd;
}

int SendMsg(int sfd, char *fmt, ...)
{
	int c, len;
	extern unsigned int alert_len;
	va_list va;
	/* 64k limit for now */
	char sb[8192<<1];

	va_start (va, fmt);
        vsnprintf (sb, sizeof(sb), fmt, va);
        va_end (va);

	len = strlen(sb);
	
	/* Don't send anything if we have no socket */
	if (sfd < 0)
		return 0;

        if ((c = send(sfd, sb, len, 0)) == -1)
        {
        	/* EPIPE means peer disconnected */
                if (errno == EPIPE)
                	return PEER_DISCONNECT;
                else
               		return SOCKET_ERR;
        }
	return c;
}

