/*
 * verify.c is a part of the KMT code base (C) Innosecc.com 2009
 * Author of Daemon: ryan@innosecc.com
 * Author of Client: sean@innosecc.com
 */


/* We constantly check the System.map and uncompressed kernel image
 * (Which are primary signatures) to make sure they have not been
 * modified. If they have, then we immediately shut down KMT and
 * notify the administrators
 */

#include "kmtd.h"

extern uint8_t sysmap_digest[16];
extern uint8_t kernelf_digest[16];
extern int global_sig_ret;

char *GetMD5_2(char *);
int FPSigs(char *system_map, char *kernel_image)
{
	int ret;
	extern struct opts opts;
	if (opts.debug)
		printf("[+] FPSigs begin\n");

	/* Can we fingerprint the System.map ? */
	ret = GetMD5(sysmap_digest, system_map);	
	if (ret < 0)
		return ERR_FP_SYSMAP;

	ret = GetMD5(kernelf_digest, kernel_image);
       	if (ret < 0)
               	return ERR_FP_KERNIMG;

	if (opts.debug)
		printf("[+] FPSigs end\n");
	return 0;
	
}

int VerifySigFP(char *system_map, char *kernel_image)
{
	int ret = 0, i;
	//char tmp[16];
	extern struct opts opts;
	char *tmp;
		
	tmp = GetMD5_2(system_map);
	if (tmp == NULL)
		return ERR_FP_SYSMAP;
	
	if (opts.debug)
        { 
		printf("Comparing the following two System.map digests:\n");
                for (i = 0; i < 16; i++)
                        printf("%.2x", (tmp[i] & 0xff));
                printf("\n");
                for (i = 0; i < 16; i++)
                        printf("%.2x", sysmap_digest[i] & 0xff);
                printf("\n");
        }

	if (memcmp(tmp, sysmap_digest, 16) != 0)
		return ERR_SYSMAP_MISMATCH;
	
        tmp = GetMD5_2(kernel_image);
	if (tmp == NULL)
		return ERR_FP_KERNIMG;

	if (opts.debug)
        {
		printf("Comparing the following two kernel image digests:\n");
                for (i = 0; i < 16; i++)
                        printf("%.2x", (tmp[i] & 0xff));
                printf("\n");
                for (i = 0; i < 16; i++)
                        printf("%.2x", kernelf_digest[i] & 0xff);
                printf("\n");
        }

	if (memcmp(tmp, kernelf_digest, 16) != 0)
                return ERR_KERNIMG_MISMATCH;

        return 0;
}

int
GetMD5 (char *digest, char *path)
{
	MD5_CTX ctx;
	int fd, i;
	struct stat st;
	uint8_t *mp;
	extern struct opts opts;
	
	if (opts.debug)
		printf("[+] GetMD5 Begin\n");

	if ((fd = open (path, O_RDONLY)) < 0)
		return -1;

	/* We are using fstat64 just incase the kernel image or System.map are over an NFS mount */
	/* and the NFS server is using 64-bit file ID's */
	if (fstat64 (fd, (struct stat64 *) &st) < 0)
		return -1;

	mp = mmap (NULL, st.st_size, PROT_READ, MAP_PRIVATE, fd, 0);
	if (mp == MAP_FAILED)
		exit (-1);

	MD5_Init (&ctx);
	MD5_Update (&ctx, (char *)mp, st.st_size);
	MD5_Final ((char *)digest, &ctx);

	if (opts.debug)
	{
		printf("[+] MD5 %s:\n", path);
		for (i = 0; i < 16; i++)
			printf("%.2x", digest[i] & 0xff);
		printf("\n");
	}
	close (fd);
	munmap (mp, st.st_size);

	if (opts.debug)
		printf("[+] GetMD5 End\n");
	
	return 0;
}

char *
GetMD5_2 (char *path)
{
        MD5_CTX ctx;
        int fd, i;
        struct stat st;
        uint8_t *mp;
        extern struct opts opts;
        char *digest = malloc(16);

        if ((fd = open (path, O_RDONLY)) < 0)
                return NULL;

        /* We are using fstat64 just incase the kernel image or System.map are over an NFS mount */
        /* and the NFS server is using 64-bit file ID's */
        if (fstat64 (fd, (struct stat64 *) &st) < 0)
                return NULL;

        mp = mmap (NULL, st.st_size, PROT_READ, MAP_PRIVATE, fd, 0);
        if (mp == MAP_FAILED)
                exit (-1);

        MD5_Init (&ctx);
        MD5_Update (&ctx, (char *)mp, st.st_size);
        MD5_Final ((char *)digest, &ctx);

        if (opts.debug)
        {
                printf("[+] MD5 %s:\n", path);
                for (i = 0; i < 16; i++)
                        printf("%.2x", digest[i] & 0xff);
		printf("Closing here?\n", GetMD5_2);
        }
        close (fd);
        munmap (mp, st.st_size);
	return digest;
}

