/*
 * KsymStat     
 * (C) innosecc.com 2009 
 * Main Author: Ryan O'Neill
 * Secondary Author: Sean Hearney
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <fcntl.h> 
#include <strings.h>
#include <string.h>
#include <sys/mman.h>
#include <getopt.h>
#include <errno.h>

#define TEXT 0xc0100000
#define BUFSZ 255
#define CODE_LEN 10
#define PAGE_SIZE 4096

/* NOTE: edi and esi could be used as well */
#define JMP_EAX 0	
#define JMP_EBX 1
#define JMP_ECX 2
#define JMP_EDX 3
#define PUSH_RET 4
#define JMP_EBP 5

#ifdef __FreeBSD__
#include <kvm.h>
#include <nlist.h>
#endif

#ifdef x86_64
typedef unsigned long kmem_off_t;
#else
typedef unsigned long long kmem_off_t;
#endif

#define USE_SYSMAP 1
#define USE_KALLSYMS 0

char * show_repl(int type, unsigned long vaddr)
{
	char *p = (char *)malloc(64);
	if (!p)
	{
		perror("Fatal error - malloc");
		exit (-1);
	}

	switch (type)
	{
	case JMP_EAX:
		sprintf(p, "movl $0x%x, \%eax\njmp *\%eax\n", vaddr);
		break;
	case JMP_EBX:
		sprintf(p, "movl $0x%x, %%ebx\njmp *%%ebx\n", vaddr);
                break;
 	case JMP_ECX:
                sprintf(p, "movl $0x%x, %%ecx\njmp *%%ecx\n", vaddr);
		break;
        case JMP_EDX:
                sprintf(p, "movl $0x%x, %%edx\njmp *%%edx\n", vaddr);
		break;
	case PUSH_RET:
		sprintf(p, "pushl $0x%x\nret\n", vaddr);
		break;
	case JMP_EBP:
		sprintf(p, "movl $0x%x, %%ebp\njmp *%%ebp\n", vaddr);
		break;
	}
	return p;
}

unsigned long get_symbol(char *name, char type, char *sysmap)
{
        FILE *fd;
        char buf[BUFSZ], symbol_s[BUFSZ];
        int i;
        unsigned long address, next_symaddr;
        char tmp[BUFSZ], tmp_type;
	char path[BUFSZ];

	if (type)
		strcpy(path, sysmap);
	else
		strcpy(path, "/proc/kallsyms");

        if ((fd = fopen(path, "r")) == NULL)
        {
  	        perror("kallsyms fopen");
                return -1;
        }

        while(!feof(fd))
        {
                bzero((char *)symbol_s, BUFSZ);
                fscanf(fd, "%x %c %s", &address, &type, &symbol_s);
                
		if (strcmp(symbol_s, name) == 0)
                {
                        fscanf(fd, "%x %c %s", &next_symaddr, &tmp_type, &tmp);
                        fclose(fd);
                        return address;
                }
                /* kallsyms isn't a regular file, meaning there is no EOF */
                if (!strcmp(symbol_s, ""))
		  	break;
        }
        return 0;
}

#ifdef __FreeBSD__
unsigned long get_symbol_fbsd(char *name)
{
        char err[255];
        kvm_t *kd;
        struct nlist nl[] = { { NULL }, { NULL }, };

        nl[0].n_name = name;

        kd = kvm_openfiles(NULL, NULL, NULL, O_RDONLY, err);
        if (!kd)
        {
                printf("get_symbol_fbsd: %s\n",err);
                return -1;
        }

        if (kvm_nlist(kd, nl) < 0)
        {
                printf("get_symbol_fbsd: %s\n",kvm_geterr(kd));
                return -1;
        }

        if(nl[0].n_value)
                return nl[0].n_value;
        else
        return -1;

}
#endif

int main(int argc, char **argv)
{
	int c, kfd, use_kmt_mem = 0, infected = 0;
	char *symbol, s[BUFSZ], type, sym_type, t[10], *sysmap;
	unsigned long symvaddr;
	unsigned long evilvaddr = 0;
	FILE *fd;
	char buf[16];
	unsigned int r;
	#ifdef __linux__
	unsigned char mem[10];
	#else
	unsigned char *mem;
	#endif
	
	int file_method = USE_KALLSYMS;

	struct opts 
	{
		char all;
		char sym;
		char sys;
	} opts;
	
	opts.all = 0;
	opts.sym = 0;
	opts.sys = 0;

 	 unsigned char jmp_eax[3];
 	         jmp_eax[0] = 0xb8;
        	 jmp_eax[1] = 0xff;
       		 jmp_eax[2] = 0xe0;

	 unsigned char jmp_ebx[3];
		 jmp_ebx[0] = 0xbb;
		 jmp_ebx[1] = 0xff;
		 jmp_ebx[2] = 0xe3;
	
	 unsigned char jmp_ecx[3];
		 jmp_ecx[0] = 0xb9;
		 jmp_ecx[1] = 0xff;
		 jmp_ecx[2] = 0xe1;
	
	 unsigned char jmp_edx[3];
		 jmp_edx[0] = 0xba;
		 jmp_edx[1] = 0xff;
		 jmp_edx[2] = 0xe2;
		
         unsigned char jmp_ebp[3];
        	 jmp_ebp[0] = 0xbd;
        	 jmp_ebp[1] = 0xff;
       	         jmp_ebp[2] = 0xe5;

         unsigned char push[2];
        	 push[0] = 0x68;
        	 push[1] = 0xc3;

	usage:
	if (argc < 2)
	{
		printf("KsymStat (C) innosecc.com\n"
		       "Usage:\t\t%s [-M] [-S <sysmap>] -a\n"
		       "\t\t%s [-M] [-S <sysmap>] -s <symbol>\n\n"
		       "[-M] Use /dev/kmt_mem (An optional replacement for /dev/kmem)\n"
		       "[-a] (note: Linux Only) Scan all symbols (Kernel functions and syscalls)\n"
		       "[-s <symbol>] Specify a symbol (Do not specify a data structure like the sys_call_table)\n"
		       "[-S <sysmap>] Use given System.map file instead of /proc/kallsyms default to locate symbol\n\n"
			"<ryan@bitlackeys.com>\n", argv[0], argv[0]);
		exit(0);  
	}

	while ((c = getopt(argc, argv, "as:MS:")) != -1)
	{
		switch(c)
		{
			case 'M':
				#ifdef __linux__
				use_kmt_mem++;
				#endif
				break;
			case 'a':
				#ifdef __FreeBSD__
				printf("This option only exists in Linux (for now)\n");
				exit(-1);
				#endif
				opts.all++;
				break;
			case 's':
				opts.sym++;
				if ((symbol = strdup(optarg)) == NULL)
				{
					 printf("[TOOL_FAILURE] Fatal error strdup()\n");
                                         exit (-1);
				}
				break;
			case 'S':
				opts.sys++;
				if ((sysmap = strdup(optarg)) == NULL)
				{
					printf("[TOOL_FAILURE] Fatal error strdup()\n");
					exit(-1);
				}
				file_method = USE_SYSMAP;
				break;
			default:
				exit(0);
		 
		}
	}
	
		
	if(opts.all == 0 && opts.sym == 0)
	{
		printf("[TOOL_FAILURE] Must use one of either -a or -s options.\n");
		exit(EXIT_FAILURE);
	}

	if(opts.all > 0 && opts.sym > 0)
	{
		printf("[TOOL_FAILURE] Cannot use -s and -a options together.\n");
		exit(EXIT_FAILURE);
	}
	
	if (opts.sym)
	{
#ifdef __FreeBSD__
        	symvaddr = get_symbol_fbsd(symbol);
#else
                if (opts.sys)
    	        	symvaddr = get_symbol(symbol, file_method, sysmap);
		else
			symvaddr = get_symbol(symbol, file_method, NULL);
#endif

	} 


	printf("\nKsymSatat (C) innosecc.com\n");
	if (use_kmt_mem)
  		printf("[+] Opening /dev/kmt_mem\n");
	else
		printf("[+] Opening /dev/kmem\n");
	
	if (use_kmt_mem)
	{
		if((kfd = open("/dev/kmt_mem", O_RDONLY)) == -1)
        	{
                	printf("[TOOL_FAILURE] kmt_mem open %s\n", strerror(errno));
                	exit(-1);
        	}
	}
	else
	{
		if((kfd = open("/dev/kmem", O_RDONLY)) == -1)
                {
                        printf("[TOOL_FAILURE] kmem open %s\n", strerror(errno));
                        exit(-1);
                }
	}
	if (opts.sym)
	{
		printf("[+] Symbol: %s resides at 0x%x\n", symbol, symvaddr);
		
		#ifdef __FreeBSD__
		mem = mmap(0, 10 + (PAGE_SIZE - (symvaddr & (PAGE_SIZE - 1))), PROT_READ, MAP_SHARED, kfd, symvaddr & (~(PAGE_SIZE-1)));
		if (mem == MAP_FAILED)
		{
			perror("mmap");
			goto done;
		}
		#else
		if(r = lseek64(kfd, (unsigned long long) symvaddr, SEEK_SET) != symvaddr)
                {
                        printf("[TOOL_FAILURE] lseek %s\n", strerror(errno));
                        exit(-1);
                }

                if (read(kfd, mem, 10) != 10)
                {
                       printf("[TOOL_FAILURE] read %s\n", strerror(errno));
                       exit(-1);
                }
		#endif
		printf("[!] First 7 bytes: %.2x %.2x %.2x %.2x %.2x %.2x %.2x\n", mem[0],mem[1],mem[2],mem[3],mem[4],mem[5],mem[6]);
		if (mem[0] == jmp_eax[0] && mem[5] == jmp_eax[1] && mem[6] == jmp_eax[2])
                {
                	evilvaddr = mem[1] + (mem[2] << 8) + (mem[3] << 16) + (mem[4] << 24);
                        type = JMP_EAX;
                }
                else
                if (mem[0] == jmp_ebp[0] && mem[5] == jmp_ebp[1] && mem[6] == jmp_ebp[2])
                {
                	evilvaddr = mem[1] + (mem[2] << 8) + (mem[3] << 16) + (mem[4] << 24);
                        type = JMP_EBP;
                }
                else
                if (mem[0] == jmp_ecx[0] && mem[5] == jmp_ecx[1] && mem[6] == jmp_ecx[2])
                {
                        evilvaddr = mem[1] + (mem[2] << 8) + (mem[3] << 16) + (mem[4] << 24);
                        type = JMP_ECX;
                }
                else
                if (mem[0] == jmp_ebx[0] && mem[5] == jmp_ebx[1] && mem[6] == jmp_ebx[2])
                {
                        evilvaddr = mem[1] + (mem[2] << 8) + (mem[3] << 16) + (mem[4] << 24);
                        type = JMP_EBX;
                }
                else
                if (mem[0] == jmp_edx[0] && mem[5] == jmp_edx[1] && mem[6] == jmp_edx[2])
                {
                	evilvaddr = mem[1] + (mem[2] << 8) + (mem[3] << 16) + (mem[4] << 24);
                        type = JMP_EDX;
                }
                else    
                if (mem[0] == push[0] && mem[5] == push[1])
                {
                	evilvaddr = mem[1] + (mem[2] << 8) + (mem[3] << 16) + (mem[4] << 24);
                        type = PUSH_RET;
                }
                /* Kernel functions are of type 'T' or 't' in System.map */
                if (evilvaddr)
                {
			printf("[ALERT_TO_DAEMON]");
                	printf("\n[Alert !]-> Symbol [%s] has been intercepted, and is being replaced by a function which resides at 0x%x.\n", s, evilvaddr);
                        printf("The initial bytes of kernel symbol %s have been replaced with the following code \n%s\nThis results in any calls "
                        "to symbol %s, actually redirecting to the function at 0x%x\n",symbol,  show_repl(type, evilvaddr), symbol, evilvaddr);
			printf("[!] Status of Kernel Symbol: Healthy!\n");
		}
		else
			printf("\n[KsymStat]-> Symbol [%s] is clean! In other words, no trampolines to an evil function appear to be present\n", symbol);
	}	
	#ifdef __linux__	
	else if (opts.all)
	{
		char path[255];

		if (opts.sys)
		{
			strncpy(path, sysmap, sizeof(path));
			path[sizeof(path) - 1] = '\0';
		}
		else
			strcpy(path, "/proc/kallsyms");
		
		printf("[+] Opening %s\n", path);
		
		if ((fd = fopen(path, "r")) == NULL)
		{
			printf("[TOOL_FAILURE] fopen %s %s\n",path, strerror(errno));
			exit(-1);
	 	}
		
		if (fgets(buf, sizeof(buf), fd) == NULL)
			goto file_err;

		if (buf[0] != 'c' && buf[1] != '0')
		{	
			file_err:
			printf("[TOOL_FAILURE] incorrect file type, must be System.map or /proc/kallsyms\n");
			exit(0);	
		}
		rewind(fd);
		while (!feof(fd))
		{
			bzero((char *)s, BUFSZ);
			fscanf(fd, "%x %c %s", &symvaddr, &sym_type, s);
			if (strcmp(s, "") == 0)
				break;
			
			/* Lets make sure that we aren't dealing with a kernel module or anything */
			/* We are primarily interested in 0xc0100000 - 0xc0700000 (approx) */
			if (symvaddr > 0xc0700000 || symvaddr < 0xc0000000)
				continue;
 				
			if((r = lseek64(kfd, (kmem_off_t)symvaddr, SEEK_SET)) == -1)
			{
				printf("lseek64: %d bytes\n",r);
				exit(-1);
			}

			if(read(kfd, mem, 10) != 10)
				continue;
                	
			if (mem[0] == jmp_eax[0] && mem[5] == jmp_eax[1] && mem[6] == jmp_eax[2])
			{
                        	evilvaddr = mem[1] + (mem[2] << 8) + (mem[3] << 16) + (mem[4] << 24);
				type = JMP_EAX;
			}
                	else
                	if (mem[0] == jmp_ebp[0] && mem[5] == jmp_ebp[1] && mem[6] == jmp_ebp[2])
                        {
				 evilvaddr = mem[1] + (mem[2] << 8) + (mem[3] << 16) + (mem[4] << 24);
				 type = JMP_EBP;
			}
               	 	else
                	if (mem[0] == jmp_ecx[0] && mem[5] == jmp_ecx[1] && mem[6] == jmp_ecx[2])
                        {
				evilvaddr = mem[1] + (mem[2] << 8) + (mem[3] << 16) + (mem[4] << 24);
                		type = JMP_ECX;
			}
			else
                	if (mem[0] == jmp_ebx[0] && mem[5] == jmp_ebx[1] && mem[6] == jmp_ebx[2])
                   	{
				evilvaddr = mem[1] + (mem[2] << 8) + (mem[3] << 16) + (mem[4] << 24);
                		type = JMP_EBX;
			}
			else
                	if (mem[0] == jmp_edx[0] && mem[5] == jmp_edx[1] && mem[6] == jmp_edx[2])
                        {
				evilvaddr = mem[1] + (mem[2] << 8) + (mem[3] << 16) + (mem[4] << 24);
				type = JMP_EDX;
			}
			else	
			if (mem[0] == push[0] && mem[5] == push[1])
			{
				evilvaddr = mem[1] + (mem[2] << 8) + (mem[3] << 16) + (mem[4] << 24);
				type = PUSH_RET;
			}
			/* Kernel functions are of type 'T' or 't' in System.map */
			if (evilvaddr && (sym_type == 'T' || sym_type == 't'))
                	{
				/* Daemon reads from stdout, so it catches the ALERT here */
				printf("[ALERT_TO_DAEMON]");
                        	printf("\n[Alert !]-> Symbol [%s] has been intercepted, and is being replaced by a function which resides at 0x%x.\n", s, evilvaddr);
				printf("The initial bytes of kernel symbol %s have been replaced with the following code \n%s\nThis results in any calls "
				"to symbol %s, actually redirecting to the function at 0x%x\n",s,  show_repl(type, evilvaddr), s, evilvaddr);
				evilvaddr = 0; 
				infected++;
			}
		}
	}
	#endif
	done:
	if (opts.all)
		if (!infected)
			printf("[+] Finished scanning all kernel functions, and syscalls.\n"
			       "[+] KsymStat found no hijacked symbols with malicious trampolines.\n"
			       "[+] Run ktraq, dps, dr_chk or kmemstat to look for other types of infections\n");	
		else
			printf("[+] Finished scanning all kernel functions, and syscalls.\n"
                               "[+] KsymStat found %d hijacked symbols with malicious trampolines.\n"
                               "[+] This indicates a probable kernel rootkit is installed!\n", infected);

	
	#ifdef __FreeBSD__
	if (opts.sym)
		munmap(mem, 10 + (PAGE_SIZE - (symvaddr & (PAGE_SIZE - 1))));
	#endif
	if (opts.sys)
		free(sysmap);
	close(kfd); 
	
}
